/*
** CellProgram.cc
*/

# include "basic.h"

CellProgram::CellProgram ()
{
  nelements = 0;
  nspace = BASESIZE; 
  elements = new CellState *[nspace];
  resolved = false;

  initialState = 0;
  finalState = 5; //!
  states = NULL;
}

CellProgram::~CellProgram ()
{
  int i;

  for (i = 0; i < nelements; i++) {
    delete elements[i];
  }

  delete elements;
}

void CellProgram::grow ()
{
  int i;
  CellState **newelements; 
  int numnew;
  
  if (nelements < BASESIZE)
    {
      numnew = BASESIZE;
    }
  else
    {
      numnew = nelements;
    }

  nspace = numnew + nspace; 

  newelements = new CellState *[nspace];
  assert (newelements != NULL);
  
  for (i = 0; i < nelements; i++)
    {
      newelements[i] = elements[i];
    }
  
  delete elements;
  elements = newelements;
}

void CellProgram::addState (CellState *trans)
{
  if (nspace == 0) grow ();
  
  assert (nspace > 0);

  nspace--;
  elements[nelements] = trans;
  nelements++;
}

void CellProgram::resolveStates ()
{
  assert (!resolved);
  states = new string [nelements];
  
  for (int i = 0; i < nelements; i++)
    {
      CellState *cs = elements[i];
      string name = cs->getName ();
      
      for (int ci = 0; ci < i; ci++) {
        if (states[ci] == name) {
          cerr << "Duplicate state declaration: " << name << endl;
          exit (1);
        }
      }
      
      states[i] = name;
    }
  
  for (int i = 0; i < nelements; i++) 
    {
      elements[i]->resolveStates (this);
    }

  nchemicals = 0;
  maxchemicals = MAX_CHEMICALS;
  chemicals = new string [maxchemicals];
  
  // Chemicals
  for (int i = 0; i < nelements; i++) 
    {
      CellState *cs = elements [i];
      cs->resolveChemicals (this);
    }

  resolved = true;
}

int CellProgram::lookupState (string state) const
{
  assert (states != NULL);

  for (int i = 0; i < nelements; i++) 
    {
      if (state == states[i]) {
        return i;
      }
    }

  return -1;
}

int CellProgram::lookupAddChemical (string chemical) 
{
  assert (chemicals != NULL);

  for (int i = 0; i < nchemicals; i++) 
    {
      if (chemical == chemicals[i]) {
        return i;
      }
    }

  assert (nchemicals + 1 < maxchemicals);

  chemicals[nchemicals] = chemical;
  return nchemicals++;
}
      
const string CellProgram::toString () const
{
  string res = "";
  char buf[BUFSIZE];

  if (nchemicals > 0) {
    res += "Chemicals: ";
    snprintf (buf, BUFSIZE, "%d", nchemicals);
    res += buf;
    res += "\n";
    
    for (int i = 0; i < nchemicals; i++) {
      snprintf (buf, BUFSIZE, "   %3d: ", i);
      res += buf;
      res += chemicals[i];
      res += "\n";
    }
  }

  for (int i = 0; i < nelements; i++)
    {
      if (i != 0) {
        res += "\n";
      }
      
      res += elements[i]->toString ();
    }
  
  return res;
}

CellState *CellProgram::getState (CellStateId current) const
{
  assert (current >= 0);
  assert (current < nelements);
  return elements[current];
}

bool CellProgram::hasStateColor (CellStateId current) const
{
  return getState (current)->hasColor ();
}

int CellProgram::getStateColor (CellStateId current) const
{
  return getState (current)->getColor ();
}

EmissionList *CellProgram::getEmissions (CellStateId current) const
{
  return getState (current)->getEmissions ();
}

void CellProgram::getTransition (CellStateId inp, Environment *env,
                                 CellStateId *next1, CellStateId *next2, t_splitKind *mode) const
{
  // cerr << "Transition from " << inp << "...";
  //  cerr << "Cell program: " << toString ();
  CellState *cs = getState (inp);

  cs->getTransition (env, next1, next2, mode);

  /*
  if (*next1 != 0) {
    if (*next2 == CELL_NULL_STATE) {
      cerr << "Transition: " << inp << " ==> " << *next1 << "\n";
    } else {
      cerr << "Transition: " << inp << " ==> " << *next1 << ", " << *next2 << "\n";
    }
  }
  */
}








