/*
** ConcentrationList.cc
*/

# include "basic.h"

ConcentrationList::ConcentrationList ()
{
  nelements = 0;
  nspace = BASESIZE; 
  elements = new ChemicalConcentration *[nspace];
}

ConcentrationList::~ConcentrationList ()
{
  int i;

  for (i = 0; i < nelements; i++) {
    delete elements[i];
  }

  delete elements;
}

void ConcentrationList::grow ()
{
  int i;
  ChemicalConcentration **newelements; 
  int numnew;
  
  if (nelements < BASESIZE)
    {
      numnew = BASESIZE;
    }
  else
    {
      numnew = nelements;
    }

  nspace = numnew + nspace; 

  newelements = new ChemicalConcentration *[nspace];
  assert (newelements != NULL);
  
  for (i = 0; i < nelements; i++)
    {
      newelements[i] = elements[i];
    }
  
  delete elements;
  elements = newelements;
}

void ConcentrationList::add (ChemicalConcentration *conc)
{
  cerr << "Adding: " << conc->toString () << endl;
    
  if (nspace == 0) grow ();
  
  assert (nspace > 0);

  nspace--;
  elements[nelements] = conc;
  nelements++;
}

const string ConcentrationList::toString () const
{
  string res = "";

  for (int i = 0; i < nelements; i++)
    {
      if (i != 0) {
	res += ", ";
      }

      res += elements[i]->toString ();
    }

  return res;
}

void ConcentrationList::resolveChemicals (CellProgram *cpa) 
{
  for (int i = 0; i < nelements; i++)
    {
      elements[i]->resolveChemicals (cpa);
    }
}

bool ConcentrationList::matchesEnvironment (Environment *env) const
{
  for (int i = 0; i < nelements; i++)
    {
      ChemicalConcentration *conc = elements[i];

      if (!conc->matchesEnvironment (env)) {
        return false;
      }
    }

  return true;
}
