/*
** EmissionList.cc
*/

# include "basic.h"

EmissionList::EmissionList ()
{
  nelements = 0;
  nspace = BASESIZE; 
  elements = new Emission *[nspace];
  current = -1;
}

EmissionList::~EmissionList ()
{
  int i;

  for (i = 0; i < nelements; i++) {
    delete elements[i];
  }

  delete elements;
}

void EmissionList::grow ()
{
  int i;
  Emission **newelements; 
  int numnew;
  
  if (nelements < BASESIZE)
    {
      numnew = BASESIZE;
    }
  else
    {
      numnew = nelements;
    }

  nspace = numnew + nspace; 

  newelements = new Emission *[nspace];
  assert (newelements != NULL);
  
  for (i = 0; i < nelements; i++)
    {
      newelements[i] = elements[i];
    }
  
  delete elements;
  elements = newelements;
}

void EmissionList::add (Emission *conc)
{
  if (nspace == 0) grow ();
  
  assert (nspace > 0);

  nspace--;
  elements[nelements] = conc;
  nelements++;
}

void EmissionList::resolveChemicals (CellProgram *cpa) 
{
  for (int i = 0; i < nelements; i++)
    {
      elements[i]->resolveChemicals (cpa);
    }
}


const string EmissionList::toString () const
{
  string res = "";

  for (int i = 0; i < nelements; i++)
    {
      if (i != 0) {
	res += ", ";
      }

      res += elements[i]->toString ();
    }

  return res;
}

void EmissionList::startIterator () 
{
  assert (current == -1);
  current = 0;
}

Emission *EmissionList::getNext ()
{
  assert (current >= 0 && current < nelements);
  return elements[current++];
}

bool EmissionList::hasMore () 
{
  return (current < nelements);
}

void EmissionList::finishIterator ()
{
  assert (current != -1);
  current = -1;
}


