#ifndef __SIMINSTRUCTION_H__
#define __SIMINSTRUCTION_H__

class SimInstruction
{
 private:
  int step;

 public:
  static SimInstruction *fromString (char *);
  virtual string toString ();
  virtual void interpret (CellSimulator *);
  int getStep () const { return step; } ;
};

class NewCellInstruction : public SimInstruction
{
 private:
  CellStateId state;
  float x;
  float y;
  float z;
  float r;
  float axis;
  float normal;

 public:
  NewCellInstruction (CellStateId state, float p_x, float p_y,
                      float p_z, float p_r, float p_axis, float p_normal) ;
  NewCellInstruction (char *) ;
  void interpret (CellSimulator *);
} ;


class RandomCellsInstruction : public SimInstruction
{
 private:
  int ncells;

  CellStateId minstate;
  CellStateId maxstate;

  float minx;
  float maxx;

  float miny;
  float maxy;

  float minz;
  float maxz;

  float minr;
  float maxr;

  float minaxis;
  float maxaxis;

  float minnormal;
  float maxnormal;

 public:
  /*
  RandomCellsInstruction (CellStateId minstate, CellStateId maxstate,
                          float p_minx, float p_maxx,
                          float p_miny, float p_maxy,
                          float p_minz, float p_maxz, 
                          float p_minr, float p_maxr, 
                          float p_minaxis, float p_maxaxis,
                          float p_minnormal, float p_maxnormal) ;
  */
  RandomCellsInstruction (char *) ;
  void interpret (CellSimulator *);
} ;

class KillInstruction : public SimInstruction
{
 private:
  float x;
  float y;
  float z;
  float r;

 public:
  KillInstruction (float p_x, float p_y, float p_z, float p_r) ;
  KillInstruction (char *) ;
  void interpret (CellSimulator *);
} ;

class StopInstruction : public SimInstruction
{
 public:
  StopInstruction ();
  void interpret (CellSimulator *);
} ;

class StoreInstruction : public SimInstruction
{
 public:
  StoreInstruction ();
  void interpret (CellSimulator *);
} ;

# endif

