/*
** cellsim.cc - main 
*/

# include <fstream>
# include "basic.h"
# include "CPAGrammar.h"

extern int yyparse (void);
extern int yydebug;
extern FILE *yyin, *yyout;

CellProgram *g_cellProgram = NULL;
CellSimulator g_simulator;
ofstream g_DumpFile("statedump.txt");

int g_tick = 0;

// this must be in threads somewhere? 
extern void Initialize(int argc,char **argv);
static bool s_batch = false;
static char *s_outputFile = NULL;

int main (int argc, char *argv[])
{
  Initialize (argc, argv);
  char *cpafile = NULL;
  char *simfile = NULL;

  for (int i = 1; i < argc; i++)
    {
      if (*argv[i] == '-')
        {
          char *arg = argv[i] + 1;

          if (strcmp (arg, "batch") == 0) {
            s_batch = true;
          } else if (strcmp (arg, "o") == 0) {

            if (i == argc - 1) {
              cerr << "Missing file for -o <file> option\n";
              exit (-1);
            } else {
              s_outputFile = argv[++i];
            }
          } else {
            cerr << "Unrecognized option (skipping): " << argv[i] << "\n";
          }
        }
      else
        {
          if (cpafile == NULL) {
            cpafile = argv[i];
          } else {
            if (simfile != NULL)
              {
                cerr << "Usage: cellsim [options] <Cell-Program-file> [<Cell-Simulation-file>]\n";
              }
            
            simfile = argv[i];
          }
        }
    }
  
  if (cpafile == NULL)
    {
      cerr << "Usage: cellsim [options] <Cell-Program-file> [<Cell-Simulation-file>]\n";
      exit(1);
    }

  yyin = fopen (cpafile, "r");
  
  if (yyin != NULL) {
    cerr << "Opened cpa file: " << cpafile << endl;
  } else {
    cerr << "Cannot open cpa file: " << cpafile << endl;
    exit (1);
  }

  // yydebug = 1;
  yyparse();

  assert (g_cellProgram != NULL);

  g_cellProgram->resolveStates (); 
  // cerr << "Loaded cell program: " << g_cellProgram->toString () << "\n"; 

  g_simulator.init (s_batch, simfile);
  
  if (!s_batch) {
    register_thread (display_thread, DISPLAY_TID);
  }

  register_thread (simulator_thread, SIMULATOR_TID);
  currentThread->Finish ();
}

void exitProgram()
{
  cerr << "Exiting...";
  
  if (!s_batch) {
    clearDisplay();
  }

  cerr << "after " << g_tick - 1 << " steps.\n";
  
  if (s_outputFile != NULL) {
    ofstream sout (s_outputFile);

    if (sout.is_open ()) {
      for (int i = 0; i < g_simulator.numCells (); i++)
        {
          sout << i << ": " << g_simulator.getCell (i)->toString () << endl;
        }
      
      sout.close ();
    } else {
      cerr << "Cannot open output file: " << s_outputFile << endl;
    }
  } else {
    /*
    cout << "CellState: (x, y, z)\n";
    for (int i = 0; i < g_simulator.numCells (); i++)
      {
        cout << i << ": " << g_simulator.getCell (i)->toString () << endl;
      }
    */
  }

  exit (0);
}



