/*
** display.cc
*/

# include "basic.h"
# include <unistd.h>
# include <X11/Xlib.h>
# include <X11/keysym.h> /*<-- contains key codes */

static Display *xdisplay;
static Bool ret;
static Window xwindow;
static Pixmap xbackbuffer;
static Pixmap xpixmap;
static int xscreen;
static unsigned long xforeground, xbackground;
static XEvent xevent;
static GC xgc;
static Colormap cmap;
bool display_enabled=true;

static void handleEvent();
static void display_all_nodes();

extern void exitProgram(); // in cellsim.cc

bool display_locked = false;
bool show_axes=false;

static void filledCircle(int x,int y,int r)
{
	XFillArc(xdisplay,xbackbuffer,xgc,x-r,y-r,r*2,r*2,0,360*64);
}

static void drawLine(int x1,int y1,int x2,int y2)
{
	XDrawLine(xdisplay,xbackbuffer,xgc,x1,y1,x2,y2);
}

static void display_all_nodes()
{
	int i;
   CellStateId s;
	int color;
	int x,y,r;
	double abAxis, abNormal;

	XCopyArea (xdisplay, xpixmap, xbackbuffer, xgc, 0, 0, W_WIDTH, W_HEIGHT, 0, 0);

	for (i = 0; i < g_simulator.numCells (); i++)
     {
       SimCell *cell = g_simulator.getCell (i);
       
       if (cell->cell_state != CELL_NULL_STATE)
         {
           s = cell->cell_state;
           x = g_simulator.getX (i);
           y = g_simulator.getY (i);
           r = g_simulator.getR (i);

           if (r == 0) {
             r = 1; // Need something to display
           }

           abAxis = cell->abAxis;
           abNormal = cell->abNormal;
           
           color = cell->getDisplayColor ();
           // cerr << "Display cell: " << cell->toString () << ": " << color << "\n";
           // cerr << "Display cell " << i << " state " << s << " at " << x << ", " << y << ", " << r << "\n";
           XSetForeground (xdisplay, xgc, color);

           filledCircle (g_simulator.getX (i), g_simulator.getY (i), 
                         max (g_simulator.getR (i), 1));
           
           if (show_axes && r > 2)
             {
               XSetForeground (xdisplay, xgc, (int) (color + (65535 / 2)) % 65535);
               drawLine(x,y,(int)(x+r*cos(abAxis)),(int)(y-r*sin(abAxis)));
               drawLine(x,y,(int)(x+r*cos(abNormal)),(int)(y-r*sin(abNormal)));
             }
         }
     }
   
	XCopyArea (xdisplay, xbackbuffer, xwindow, xgc, 0, 0, W_WIDTH, W_HEIGHT, 0, 0);
}

static void handleEvent()
{
	switch (xevent.type)
	{
		/* ignore this event */
		case Expose:
			break;
		case ButtonPress:
			/* x=xevent.xbutton.x;					
			y=xevent.xbutton.y; */
			break;
		case KeyPress:
			switch (XLookupKeysym(&xevent.xkey,0))
           {
           case XK_A:
           case XK_a:
             show_axes = !show_axes;
             break;
           case XK_space: 
             display_locked = false;
             break;
           case XK_q: 
           case XK_Q:
           case XK_Escape: 
             exitProgram();
             break;
             /* /usr/include/X11/keysymdef.h <-- contains key codes */
           }
	}
}

void display_thread (int dummy)
{
  int depth;
  
  Window root_win;
  /* connect to the X server */
  xdisplay = XOpenDisplay ("");
  display_enabled=true;
  if (xdisplay == NULL) {
    fprintf (stderr, "cannot connect to server\n");
    exit (EXIT_FAILURE);
  }
  root_win = DefaultRootWindow(xdisplay);
  xscreen = DefaultScreen (xdisplay);
  xbackground = BlackPixel (xdisplay, xscreen);
  xforeground = WhitePixel (xdisplay, xscreen);
  xwindow = XCreateSimpleWindow (xdisplay, root_win, 0, 0, W_WIDTH, W_HEIGHT, 2, xforeground, xbackground);
  if (!xwindow) {
    fprintf (stderr, "cannot open window\n");
    exit (EXIT_FAILURE);
  }
  depth = DefaultDepth (xdisplay, xscreen);

  xbackbuffer= XCreatePixmap (xdisplay, root_win, W_WIDTH , W_HEIGHT, depth);
  xpixmap = XCreatePixmap (xdisplay, root_win, W_WIDTH , W_HEIGHT, depth);

  if (!xbackbuffer || !xpixmap) {
    fprintf (stderr, "cannot open backbuffer/pixmap\n");
    exit (EXIT_FAILURE);
  }
  
  /* ask for exposure event */
  XSelectInput(xdisplay, xwindow, ButtonPressMask | KeyPressMask | ExposureMask);
  /* pop this window up on the screen */
  XMapRaised (xdisplay, xwindow);
  /* set graphics context of rectangle to red */
  xgc= XCreateGC (xdisplay, xwindow, 0, 0);
  cmap = DefaultColormap (xdisplay,xscreen);
  
  do
    {
      ret=XCheckWindowEvent(xdisplay,xwindow,0xfff,&xevent);
      /* third param - event mask. no idea what it is */
      
      if(ret==true)
        {
          handleEvent();
        }

      if (!display_locked)
        {
          YIELD;

          display_all_nodes ();
          display_locked = true;
        }

      /* This doesn't quite work...something about the color depth, file format?
      // if saving
      if (true)
        {
          char fname[BUFSIZE];
          snprintf (fname, BUFSIZE, "out%d.bmp", g_tick);
          XWriteBitmapFile (xdisplay, fname, xbackbuffer, W_WIDTH, W_HEIGHT, -1, -1);
        }
      */

    } while (1);
  return;
}

void clearDisplay(void)
{
	if (display_enabled)
	{
		XFreePixmap(xdisplay,xbackbuffer);
		XFreeGC(xdisplay,xgc);
		XCloseDisplay(xdisplay);
	}
}

